# -*- coding: utf-8 -*-

"""Enforce HTTP Strict Transport Security (see e.g.: `#`_).
  
  _`#`: http://www.imperialviolet.org/2012/07/19/hope9talk.html
"""

# This implementation has been extracted to the `pyramid_hsts` dependency.
# The import api is provided here for backwards compatibility.
from pyramid_hsts import *
